@@ -33,6 +33,7 @@ using GMime;
 
 using Beagle;
 using Beagle.Daemon;
+using Beagle.Util;
 
 namespace Beagle.Filters {
 
@@ -40,13 +41,15 @@ namespace Beagle.Filters {
 
 		private static bool gmime_initialized = false;
 
-		private GMime.Stream stream;
-		private GMime.Parser parser;
 		private GMime.Message message;
 		private PartHandler handler;
 
 		public FilterMail ()
 		{
+			// 1: Make email addresses non-keyword, add sanitized version
+			//    for eaching for parts of an email address.
+			SetVersion (1);
+
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("message/rfc822"));
 		}
 
@@ -62,19 +65,36 @@ namespace Beagle.Filters {
 				}
 			}
 
-			int mail_fd = Mono.Unix.Syscall.open (info.FullName, Mono.Unix.OpenFlags.O_RDONLY);
+			int mail_fd = Mono.Unix.Native.Syscall.open (info.FullName, Mono.Unix.Native.OpenFlags.O_RDONLY);
 			
 			if (mail_fd == -1)
 				throw new IOException (String.Format ("Unable to read {0} for parsing mail", info.FullName));
 
-			this.stream = new GMime.StreamFs (mail_fd);
-			this.parser = new GMime.Parser (stream);
+			GMime.StreamFs stream = new GMime.StreamFs (mail_fd);
+			GMime.Parser parser = new GMime.Parser (stream);
+			this.message = parser.ConstructMessage ();
+			stream.Dispose ();
+			parser.Dispose ();
+
+			if (this.message == null)
+				Error ();
+		}
+
+		private bool HasAttachments (GMime.Object mime_part)
+		{
+			if (mime_part is GMime.MessagePart)
+				return true;
+
+			// Messages that are multipart/alternative shouldn't be considered as having
+			// attachments.  Unless of course they do.
+			if (mime_part is GMime.Multipart && mime_part.ContentType.Subtype.ToLower () != "alternative")
+				return true;
+
+			return false;
 		}
 
 		protected override void DoPullProperties ()
 		{
-			this.message = this.parser.ConstructMessage ();
-
 			string subject = GMime.Utils.HeaderDecodePhrase (this.message.Subject);
 			AddProperty (Property.New ("dc:title", subject));
 
@@ -84,7 +104,10 @@ namespace Beagle.Filters {
 			addrs = this.message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
 				AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
-				AddProperty (Property.NewKeyword ("fixme:to_address", ia.Addr));
+				if (ia.AddressType != GMime.InternetAddressType.Group) {
+					AddProperty (Property.New ("fixme:to_address", ia.Addr));
+					AddProperty (Property.NewUnstored ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+				}
 				AddProperty (Property.New ("fixme:to_name", ia.Name));
 			}
 			addrs.Dispose ();
@@ -92,7 +115,10 @@ namespace Beagle.Filters {
 			addrs = this.message.GetRecipients (GMime.Message.RecipientType.Cc);
 			foreach (GMime.InternetAddress ia in addrs) {
 				AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
-				AddProperty (Property.NewKeyword ("fixme:cc_address", ia.Addr));
+				if (ia.AddressType != GMime.InternetAddressType.Group) {
+					AddProperty (Property.New ("fixme:cc_address", ia.Addr));
+					AddProperty (Property.NewUnstored ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+				}
 				AddProperty (Property.New ("fixme:cc_name", ia.Name));
 			}
 			addrs.Dispose ();
@@ -100,20 +126,46 @@ namespace Beagle.Filters {
 			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (this.message.Sender));
 			foreach (GMime.InternetAddress ia in addrs) {
 				AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
-				AddProperty (Property.NewKeyword ("fixme:from_address", ia.Addr));
+				if (ia.AddressType != GMime.InternetAddressType.Group) {
+					AddProperty (Property.New ("fixme:from_address", ia.Addr));
+					AddProperty (Property.NewUnstored ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+				}
 				AddProperty (Property.New ("fixme:from_name", ia.Name));
 			}
 			addrs.Dispose ();
 
-			if (this.message.MimePart is GMime.Multipart || this.message.MimePart is GMime.MessagePart)
+			if (HasAttachments (this.message.MimePart))
 				AddProperty (Property.NewFlag ("fixme:hasAttachments"));
 
-			string list_id = this.message.GetHeader ("List-Id");
+			// Store the message ID and references are unsearched
+			// properties.  They will be used to generate
+			// conversations in the frontend.
+			string msgid = this.message.GetHeader ("Message-Id");
+		        if (msgid != null)
+				AddProperty (Property.NewUnsearched ("fixme:msgid", GMime.Utils.DecodeMessageId (msgid)));
 
+			foreach (GMime.References refs in this.message.References)
+				AddProperty (Property.NewUnsearched ("fixme:reference", refs.Msgid));
+
+			string list_id = this.message.GetHeader ("List-Id");
 			if (list_id != null) {
 				// FIXME: Might need some additional parsing.
 				AddProperty (Property.NewKeyword ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
 			}
+
+			// KMail can store replies in the same folder
+			// Use issent flag to distinguish between incoming
+			// and outgoing message
+			string kmail_msg_sent = this.message.GetHeader ("X-KMail-Link-Type");
+			bool issent_is_set = false;
+			foreach (Property property in IndexableProperties) {
+				if (property.Key == "fixme:isSent") {
+					issent_is_set = true;
+					break;
+				}
+			}
+			if (!issent_is_set && kmail_msg_sent != null && kmail_msg_sent == "reply")
+				AddProperty (Property.NewFlag ("fixme:isSent"));
 		}
 
 		protected override void DoPullSetup ()
@@ -147,16 +199,14 @@ namespace Beagle.Filters {
 		
 		public void Dispose ()
 		{
-			if (this.message != null)
+			if (this.handler != null && this.handler.Reader != null)
+				this.handler.Reader.Close ();
+			this.handler = null;
+
+			if (this.message != null) {
 				this.message.Dispose ();
-
-			if (this.stream != null) {
-				this.stream.Close ();
-				this.stream.Dispose ();
+				this.message = null;
 			}
-
-			if (this.parser != null)
-				this.parser.Dispose ();
 		}
 
 		private class PartHandler {
@@ -171,37 +221,75 @@ namespace Beagle.Filters {
 				this.filter = filter;
 			}
 
-			public void OnEachPart (GMime.Object part)
+			private bool IsMimeTypeHandled (string mime_type)
 			{
-				//for (int i = 0; i < depth; i++)
+				foreach (FilterFlavor flavor in FilterFlavor.Flavors) {
+					if (flavor.IsMatch (null, null, mime_type.ToLower ()))
+						return true;
+				}
+
+				return false;
+			}
+
+			public void OnEachPart (GMime.Object mime_part)
+			{
+				GMime.Object part = null;
+				bool part_needs_dispose = false;
+
+				//for (int i = 0; i < this.depth; i++)
 				//  Console.Write ("  ");
-				//Console.WriteLine ("Content-Type: {0}", part.ContentType);
+				//Console.WriteLine ("Content-Type: {0}", mime_part.ContentType);
 			
 				++depth;
 
-				if (part is GMime.MessagePart) {
-					GMime.MessagePart msg_part = (GMime.MessagePart) part;
+				if (mime_part is GMime.MessagePart) {
+					GMime.MessagePart msg_part = (GMime.MessagePart) mime_part;
 
 					using (GMime.Message message = msg_part.Message) {
 						using (GMime.Object subpart = message.MimePart)
 							this.OnEachPart (subpart);
 					}
-				} else if (part is GMime.Multipart) {
-					GMime.Multipart multipart = (GMime.Multipart) part;
+				} else if (mime_part is GMime.Multipart) {
+					GMime.Multipart multipart = (GMime.Multipart) mime_part;
 
 					int num_parts = multipart.Number;
-					for (int i = 0; i < num_parts; i++) {
-						using (GMime.Object subpart = multipart.GetPart (i))
-							this.OnEachPart (subpart);
+
+					// If the mimetype is multipart/alternative, we only want to index
+					// one part -- the richest one we can filter.
+					if (mime_part.ContentType.Subtype.ToLower () == "alternative") {
+						// The richest formats are at the end, so work from there
+						// backward.
+						for (int i = num_parts - 1; i >= 0; i--) {
+							GMime.Object subpart = multipart.GetPart (i);
+
+							if (IsMimeTypeHandled (subpart.ContentType.ToString ())) {
+								part = subpart;
+								part_needs_dispose = true;
+								break;
+							} else {
+								subpart.Dispose ();
+							}
+						}
 					}
-				} else if (part is GMime.Part) {
-					MemoryStream stream = null;
+
+					// If it's not alternative, or we don't know how to filter any of
+					// the parts, treat them like a bunch of attachments.
+					if (part == null) {
+						for (int i = 0; i < num_parts; i++) {
+							using (GMime.Object subpart = multipart.GetPart (i))
+								this.OnEachPart (subpart);
+						}
+					}
+				} else if (mime_part is GMime.Part)
+					part = mime_part;
+				else
+					throw new Exception (String.Format ("Unknown part type: {0}", part.GetType ()));
+
+				if (part != null) {
+					System.IO.Stream stream = null;
 					
-					using (GMime.DataWrapper content_obj = ((GMime.Part) part).ContentObject) {
-						stream = new MemoryStream ();
-						content_obj.WriteToStream (stream);
-						stream.Seek (0, SeekOrigin.Begin);
-					}
+					using (GMime.DataWrapper content_obj = ((GMime.Part) part).ContentObject)
+						stream = content_obj.Stream;
 
 					// If this is the only part and it's plain text, we
 					// want to just attach it to our filter instead of
@@ -235,10 +323,11 @@ namespace Beagle.Filters {
 					}
 
 					this.count++;
-				} else {
-					throw new Exception (String.Format ("Unknown part type: {0}", part.GetType ()));
 				}
 
+				if (part_needs_dispose)
+					part.Dispose ();
+
 				--depth;
 			}
 
