@@ -238,7 +238,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				
 				Logger.Log.Debug ("{0}: Finished indexing {1} messages", this.folder_name, this.indexed_count);
 
-				this.MboxLastOffset = offset;
+				if (offset >= 0)
+					this.MboxLastOffset = offset;
 				this.CrawlFinished ();
 
 				return false;
@@ -249,8 +250,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		public override Indexable GetNextIndexable ()
 		{
 			using (GMime.Message message = this.mbox_parser.ConstructMessage ()) {
-				++this.count;
-
 				// Work around what I think is a bug in GMime: If you
 				// have a zero-byte file or seek to the end of a
 				// file, parser.Eos () will return true until it
@@ -261,9 +260,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				//
 				// Check if its empty by seeing if the Headers
 				// property is null or empty.
-				if (message.Headers == null || message.Headers == "")
+				if (message == null || message.Headers == null || message.Headers == "")
 					return null;
 				
+				++this.count;
+
 				string x_evolution = message.GetHeader ("X-Evolution");
 				if (x_evolution == null || x_evolution == "") {
 					Logger.Log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
@@ -323,21 +324,21 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (this.folder_name == "Sent")
+				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.Cc);
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (this.folder_name == "Sent")
+				if (this.folder_name == "Sent"  && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (message.Sender));
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (this.folder_name != "Sent")
+				if (this.folder_name != "Sent"  && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
@@ -376,8 +377,12 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			long offset = -1;
 
-			if (this.mbox_parser != null)
-				this.MboxLastOffset = offset = this.mbox_parser.FromOffset;
+			if (this.mbox_parser != null) {
+				offset = this.mbox_parser.FromOffset;
+
+				if (offset >= 0)
+					this.MboxLastOffset = offset;
+			}
 
 			string progress = "";
 			if (this.file_size > 0 && offset > 0) {
@@ -780,12 +785,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
-					indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
-					indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					if (ia.AddressType != GMime.InternetAddressType.Group) {
+						indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
+						indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					}
 					indexable.AddProperty (Property.New ("fixme:to_name", ia.Name));
 				}
 
-				if (this.folder_name == "Sent")
+				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
@@ -794,12 +801,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
-					indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
-					indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					if (ia.AddressType != GMime.InternetAddressType.Group) {
+						indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
+						indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					}
 					indexable.AddProperty (Property.New ("fixme:cc_name", ia.Name));
 				}
 
-				if (this.folder_name == "Sent")
+				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
@@ -808,12 +817,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
-					indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
-					indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					if (ia.AddressType != GMime.InternetAddressType.Group) {
+						indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
+						indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					}
 					indexable.AddProperty (Property.New ("fixme:from_name", ia.Name));
 				}
 
-				if (this.folder_name != "Sent")
+				if (this.folder_name != "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
 					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
