@@ -258,8 +258,10 @@ namespace Beagle.Daemon {
 			DirectoryInfo lock_dir_info;
 			lock_dir_info = new DirectoryInfo (LockDirectory);
 			foreach (FileInfo info in lock_dir_info.GetFiles ()) {
-				if (IsDanglingLock (info))
+				if (IsDanglingLock (info)) {
+					Logger.Log.Warn ("Found a dangling index lock on {0}", info.FullName);
 					return false;
+				}
 			}
 
 			return true;
@@ -776,6 +778,7 @@ namespace Beagle.Daemon {
 				IndexReader reader;
 				reader = GetReader (PrimaryStore);
 				last_item_count = reader.NumDocs ();
+				ReleaseReader (reader);
 			}
 			return last_item_count;
 		}
@@ -1279,7 +1282,9 @@ namespace Beagle.Daemon {
 				foreach (QueryPart  sub_part in part.SubParts) {
 					LNS.Query p_subq, s_subq;
 					HitFilter sub_hit_filter; // FIXME: This is (and must be) ignored
-					// FIXME: We don't handle dates correctly for OR at all.
+					// FIXME: Any subpart in an OR which has a hit filter won't work
+					// correctly, because we can't tell which part of an OR we matched
+					// against to filter correctly.  This affects date range queries.
 					QueryPartToQuery (sub_part, only_build_primary_query,
 							  term_list,
 							  out p_subq, out s_subq, out sub_hit_filter);
@@ -1400,15 +1405,18 @@ namespace Beagle.Daemon {
 
 			public IndexReader Reader;
 			public long Version;
+			public int Refcount;
 
 			public ReaderAndVersion (IndexReader reader, long version)
 			{
 				this.Reader = reader;
 				this.Version = version;
+				this.Refcount = 1;
 			}
 		}
 
 		static private Hashtable directory_rav_map = new Hashtable ();
+		static private Hashtable reader_rav_map = new Hashtable ();
 
 		static public LNS.IndexSearcher GetSearcher (Lucene.Net.Store.Directory directory)
 		{
@@ -1422,7 +1430,7 @@ namespace Beagle.Daemon {
 			IndexReader reader;
 			long version;
 
-			lock (directory_rav_map) {
+			lock (reader_rav_map) {
 				ReaderAndVersion rav = (ReaderAndVersion) directory_rav_map [directory];
 
 				if (rav == null) {
@@ -1430,8 +1438,10 @@ namespace Beagle.Daemon {
 					reader = IndexReader.Open (directory);
 
 					rav = new ReaderAndVersion (reader, version);
+					rav.Refcount++;
 
 					directory_rav_map [directory] = rav;
+					reader_rav_map [reader] = rav;
 
 					return reader;
 				}
@@ -1439,15 +1449,49 @@ namespace Beagle.Daemon {
 				version = IndexReader.GetCurrentVersion (directory);
 				
 				if (version != rav.Version) {
-					rav.Version = version;
-					rav.Reader.Close ();
-					rav.Reader = IndexReader.Open (directory);
-				}
-				
+					UnrefReaderAndVersion_Unlocked (rav);
+
+					reader = IndexReader.Open (directory);
+
+					rav = new ReaderAndVersion (reader, version);
+					rav.Refcount++;
+
+					directory_rav_map [directory] = rav;
+					reader_rav_map [reader] = rav;
+				} else
+					rav.Refcount++;
+
 				return rav.Reader;
 			}
 		}
 
+		static private void UnrefReaderAndVersion_Unlocked (ReaderAndVersion rav)
+		{
+			rav.Refcount--;
+
+			if (rav.Refcount == 0) {
+				rav.Reader.Close ();
+				reader_rav_map.Remove (rav.Reader);
+			}
+		}
+
+		static public void ReleaseReader (IndexReader reader)
+		{
+			lock (reader_rav_map) {
+				ReaderAndVersion rav = (ReaderAndVersion) reader_rav_map [reader];
+
+				UnrefReaderAndVersion_Unlocked (rav);
+			}
+		}
+
+		static public void ReleaseSearcher (LNS.IndexSearcher searcher)
+		{
+			IndexReader reader = searcher.GetIndexReader ();
+
+			searcher.Close ();
+			ReleaseReader (reader);
+		}
+
 		///////////////////////////////////////////////////////////////////////////////////
 
 		//
@@ -1523,6 +1567,9 @@ namespace Beagle.Daemon {
 				searcher.Close ();
 			}
 
+			ReleaseReader (primary_reader);
+			ReleaseReader (secondary_reader);
+
 			// Now assemble the hits
 			int j = 0;
 			foreach (Uri uri in primary_docs.Keys) {
@@ -1598,6 +1645,9 @@ namespace Beagle.Daemon {
 				}
 			}
 
+			ReleaseReader (primary_reader);
+			ReleaseReader (secondary_reader);
+
 			return all_hits;
 		}
 	}
